% This function is used to calculate an approximation to the 
% Differential equation we are given using a set of Reduced
% basis vectors that were generated using svd
% The ODE test problem that we are trying to solve is given by:
% -beta*u'' + gamma*u' + alpha*u = f(x)

function [uh,x,PSI,U] = POD_simulate(N,level,dt,T,eta,beta,gamma,alpha,BASIS,nB,plotyes)

    close all

	% First get the geometry of the problem
	[xl,xr,ul,ur] = geometry(0);
    
	% Check to make sure the user(you) isnt an idiot
    % And change this to != if in octave, ~= if in MATLAB
	if(mod(xr-xl,(1/(2^level))) ~= 0)
		error('The size of the boundary is not consistent with the Discretization used');
    end
    
    % Information about the time stepping
    % If eta=0 then do not step through time
    if(eta == 0)
        nt = 1;
    else
        nt = floor(T./dt);
    end
    
	% Define the grid of points that we are going to use
	x = (xl:(1/(2^level)):xr)';
	
	% The number of x points we need to worry about
	nx = length(x);
    
    % SOme of the sizes that we will be concerned about
    nO = (2.*(N-1)) - 1;
	midO = ceil(nO./2);
    sideO = midO - 1;
    
	% if BASIS is a file we have to read it in first
	if(exist(BASIS) == 2)
		% Read in the basis file and set it as a matrix
		BASIS = dlmread(BASIS);
    end
    
	% Get the size of the matrix aka the number of basis functions
	BASIS = [ul.*ones(1,nB); BASIS(1:nx-2,1:nB); ur.*ones(1,nB)];
	
	% Determine the Connection coefficients
    
    % for the double derivative
	[temp,temp,Omega2] = connection_coeffs(N,2,0,level);
    
    % for the single derivative
	[temp,temp,Omega1] = connection_coeffs(N,1,0,level);
    
    Omega1 = Omega1';
    Omega2 = Omega2';
    
    % Since the first term is -u'' multiply Omega by -1
    %Omega = -beta.*Omega2 + gamma.*Omega1;
	%Omega = Omega';
	
	% Initialize the PSI matrix
	PSI = zeros(nB,nB);
    
    % Initialize the b vector
    b = zeros(nB,1);
	
	% Loop over the i reduced basis functions
	for i=1:nB
	
		% And now over the j basis functions
		for j=1:nB
			
            % This keeps track of where to start
            
            %
            
			% Now loop over all the points in our domain to calculate
			% The inner products of our basis
			for k=1:nx
				
                % If we are near the left boundary
                if(k < midO)
                    % Compute the temp sum
                    %temp = sum(BASIS(1:(k+sideO),i)'.*BASIS(1:(k+sideO),j)'.*Omega((midO+1-k):nO));
                    temp1 = BASIS(k,i)'.*sum(BASIS(1:(k+sideO),j)'.*Omega1((midO+1-k):nO));
                    temp2 = BASIS(k,i)'.*sum(BASIS(1:(k+sideO),j)'.*Omega2((midO+1-k):nO));
                elseif(k > (nx-sideO))
                    %temp = sum(BASIS((k-sideO):nx,i)'.*BASIS((k-sideO):nx,j)'.*Omega(1:(nO-(k-(nx-sideO)))));
%                     BASIS((k-sideO):nx,j)'
%                     Omega1(1:(nO-(k-(nx-sideO))))
%                     asdf
                    temp1 = BASIS(k,i)'.*sum(BASIS((k-sideO):nx,j)'.*Omega1(1:(nO-(k-(nx-sideO)))));
                    temp2 = BASIS(k,i)'.*sum(BASIS((k-sideO):nx,j)'.*Omega2(1:(nO-(k-(nx-sideO)))));
                else
                    %temp = sum(BASIS((k-sideO):(k+sideO),i)'.*BASIS((k-sideO):(k+sideO),j)'.*Omega);
                    temp1 = BASIS(k,i)'.*sum(BASIS((k-sideO):(k+sideO),j)'.*Omega1);
                    temp2 = BASIS(k,i)'.*sum(BASIS((k-sideO):(k+sideO),j)'.*Omega2);
                end
                
                % The Derivative term
                % Note the factor in front is stuff that was brought out of
                % the scaling function sums, can be changed depending on
                % the problem we are trying to solve
                %PSI(i,j) = PSI(i,j) - (2.^(level.*(1+2))).*temp;
                % Excuse the notation, I meant to have it PSI(i,j) but
                % someh
                PSI(i,j) = PSI(i,j) - beta.*temp2 + gamma.*temp1;
                
                % This accounts for the non-derivative term
                if(i == j)
                    %PSI(i,j) = PSI(i,j) + alpha.*(2.^level).*BASIS(k,i).*BASIS(k,j);
                    PSI(i,j) = PSI(i,j) + alpha.*BASIS(k,i).*BASIS(k,j);
                    if(eta ~= 0)
                        PSI(i,j) = PSI(i,j) + (eta./dt).*BASIS(k,i).*BASIS(k,j);
                    end
                        
                end
            end
            %PSI
            
        end % End looping over the j basis functions
        
        % And then create the right hand side
        %b(i) = sum((2.^(level./2)).*rhs(x).*BASIS(:,j));
        %b(i) = sum(rhs(x).*BASIS(:,i));
        
    end % End looping over the i basis functions

    % Initially time is at zero
    t = 0;
    % And the intial condition is zero
    uh = zeros(nx,1);
    
    % First compute the solution using the wavelet method so we have
    % something to compare to
   [~,~,~,~,U,~] = wavelet_ode(N,level,dt,T,eta,beta,gamma,alpha,0);
    
    % Determine the time instances
    time_instances = 0:dt:T;
    nt = length(time_instances)-1;
    quartert = ceil(nt./4);
    halft = floor(nt./2);
    quartert3 = ceil(nt.*.75);
    
    % Now we must loop over the time that we are trying to solve this for
    for i=1:nt
        
        t = t + dt;
        
        % First generate the right hand side of the equation
        for j=1:nB
            b(j) = sum(rhs(x,t).*BASIS(:,j));
            if(eta ~= 0)
                b(j) = b(j) + (eta./dt).*sum(uh.*BASIS(:,j));
            end
        end
    
        % Now find the solution
        %if(sum(b) == 0)
            
        Cj = PSI \ b;
        
        % Modify the scaling terms
%         for j=1:nB
%             Cj(j) = Cj(j)./j;
%         end
	
        % Initialize the solution vector
        uh = zeros(nx,1);
    
        % Now uh is the combination of our Cj's and our basis functions
        for j=1:nB
            uh = uh + Cj(j).*BASIS(:,j);
        end
    
        if(plotyes == 1)
            if((i==1) || (i == nt) || (i == halft) || (i == quartert) || (i == quartert3))
                i;
                % For the test problem
                %filename = ['Snapshots_D',num2str(N),'_L',num2str(level),'.txt']
                %us = dlmread(filename);
                figure(1)
                hold on
                plot(x,uh,'*-','LineWidth',1.5);
                plot(x,[ul;U(:,i);ur],'r','LineWidth',1.5);
                %plot(x,[0;us(:,1);0],'r')
                hold off
            end
        end
    end
    
    if(plotyes == 1)
        set(gca,'FontSize',16)
        xlabel('x','FontSize',16);
        ylabel('u(x,t)','FontSize',16);
        legend('u_{ROM}','u_{wavelet}');
        title(strcat('Reduced Order solution'),'FontSize',16);
    end

end