function [T] = form_T_matrix(N)

% First get the coefficients based on our value of N
h = daub_coefficients(N);

% Since John stores these coefficients consistent with Ingrid, we need to
% re-normalize them by dividing by the sqrt(2)
h = h./sqrt(2);

% Size of the matrix
n = (2.*(N - 1)) - 1;
midn = ceil(n/2);

% Number of values to calculate
m = (2.*(N - 1)) + 1;
midm = ceil(m/2);

% These are the values that will be put into the T matrix
r = zeros(1,m);

% Initialize the T matrix
T = zeros(n);

% First go through and compute all the r values.  we only need to compute
% half plus one of them because the lower half is equal to the upper half.

% dk is the step size that we take when we multiply h's and it decreases
% every i step
dk = N-1;

for i=1:midm    
    for k=1:i
        r(i) = r(i) + h(k).*h(k+dk);
    end
    dk = dk - 1;
end

for i=1:(midm-1)
    r(m+1-i) = r(i);    
end

% Now we need to formulate the T matrix

% The shift for each of our rows
dk = 2;

% Put in the middle row
T(midn,:) = r(2:m-1);

for i=1:(midn-1)
    
    % Get the upper half of the matrix
    T(midn-i,1:n+1-dk) = r(2+dk:m);
    
    % Get the lower half of the matrix
    T(midn+i,dk:n) = r(1:m-1-dk);

    % Increment dk
    dk = dk+2;

end


end