% This function is designed to calculate the connection coefficients that
% define the coefficients of the inner product calculation between scaling
% functions

function [A,b,Omega,M] = connection_coeffs(N,d1,d2,level)

    % Calculate the T matrix using the value N
    T = form_T_matrix(N);
    
    % d is the sum of d1 and d2
    d = d1 + d2;
    
    % Pull in the daubechies coefficients and store them as a values which
    % should sum to 2
    a = sqrt(2).*daub_coefficients(N);
    
    % The Moment vector fits into the row below the T-.5I matrix
    M = zeros(d,size(T,2));
    
    % M0 is the vector of moments where i=0
    M0 = zeros(d+1,1);
    M0(1) = 1;
    
    % First calculate all the M0 elements so that we can use them to
    % calculate the M^k_i elements
%     for k=1:d  
%         for i=0:N-1       
%             temp = 0; 
%             for j=0:k-1               
%                 temp = temp + nchoosek(k,j).*(i^(k-j)).*M0(j+1);
%             end                        
%             M0(k+1) = M0(k+1) + temp.*a(i+1);            
%         end
%         M0(k+1) = (1./(2.*((2^k)-1))).*M0(k+1);
%     end
    
    % This uses Nielsons notation and may be a little different? Giving us
    % better condition numbers?
    for p=1:d  
        for n=0:p-1       
            temp = 0; 
            for k=0:N-1              
                temp = temp + a(k+1).*(k.^(p-n));
            end                        
            M0(p+1) = M0(p+1) + nchoosek(p,n).*M0(n+1).*temp;            
        end
        M0(p+1) = (sqrt(2)./(2.*((2^p)-1))).*M0(p+1);
    end
    
    % Set the first value of M as the last value of M0
    %M(1) = M0(d);
    
    % Formulate the b vector
    b = zeros(size(T,2)+1,1);
    %b = zeros((2.*N)-3+d,1);
    
    % Find the middle of the Moment vector
    N_mid = N-1;
    
    % Put the value of M0 in M middle
    M(:,N_mid) = M0(2:length(M0));
    
    % Now that We have the M_0 terms we can find the other moments so that
    % we end up with M^d_i
    for k=1:d
        for i=1:N-2    
            for j=0:k       
                M(k,N_mid-i) = M(k,N_mid-i) + nchoosek(k,j).*((-i).^(k-j)).*M0(j+1);
                M(k,N_mid+i) = M(k,N_mid+i) + nchoosek(k,j).*(i.^(k-j)).*M0(j+1);
            end
            
%              M(k,N_mid-i) = M(k,N_mid-i)./(2^(level.*(k+.5)));
%              M(k,N_mid+i) = M(k,N_mid+i)./(2^(level.*(k+.5)));
            
        end
        
       M(k,:) = M(k,:)./(2^(level.*(k+.5)));
        
        %b((2.*N)-3+k) = factorial(k);
        
    end
    
    % Calculate the moments M^p_l using Nielsons notation
    %for p=1:d

    %[V,D] = eig((T-((1/(2^(d-1))).*eye(size(T,2)))));
    
    % Now put the moment condition into the T matrix
    A = [(T-((1/(2^(d+1))).*eye(size(T,2)))) ; M(d,:)];
    
    % Set the b vector
    b(size(T,2)+1) = d./(2^(level/2));
%     for i=1:d
%         b(size(T,2)+1) = b(size(T,2)+1).*(1./(2^(level.*(d+.5))));
%     end
    
    % Calculate the conection coefficients
    Omega = A \ b;
    
end

    
    