
function [cc,sucess] = connection_coeffs3(N,~,d2,d3,level)

%clear
%clc
%close all

% N = 6;
% d1 = 0;
% d2 = 1;
% d3 = 0;
% level = 4;

% Stuff that is needed.  Will compress later
N21 = (2*N) - 1;
nm1 = N - 1;
nm2 = N - 2;
d = d2 + d3;

% This adds in the size of the T matrix plus some additional rows to
% account for the moment conditions
nT = (3.*(N.^2)) - (9*N) + 7;
T = zeros(nT+d+1,nT);

% Get the daubechies coefficients
h = sqrt(2).*daub_coefficients(N);

% This matrix stores the possible scaling function combinations
A = zeros(N21,N21);

% Construct the T matrix
% Use the equation:
% A(i,j) = sum_{k=1}^{min(i,j)} h(k)*h(k+i)*h(k+j)
% this creates the Circulant matrix that we will use
% To construct T.
% Note this code is borrowed from Juan Restrepo's n-tuple code


% Loop over the possible shifts and decide which ones are possible.  Store
% the possible ones in the A matrix
for q=-nm1:nm1
    if(q > 0) 
        p1q = q;
        p2q = nm1;
    else
        p1q = 0;
        p2q = nm1 + q;
    end

    for r=-nm1:nm1
        if(r > 0)
           p1r = r;
           p2r = nm1;
        else
           p1r = 0;
           p2r = nm1 + r;
        end

        P1 = max(p1q,p1r);
        P2 = min(p2q,p2r);

        sum = 0;
        for pp=P1:P2
           sum = sum + h(pp+1)*h(pp-q+1)*h(pp-r+1);
        end
        A(q+N,r+N) = sum;
    end
end

% Once we have the circulant matrix, we need to put the values into the T
% matrix in order to determine the connection coefficients.  Note that this
% matrix of size:
%                3N^2 - 9N + 7
% which represents the number of non-zero connection coefficients that
% exist for a stationary scaling function with shifted scaling functions
% \phi _l and \phi _m 

% initialize some values
row = 0;

% Now that we have all the possible combinations we want to put their
% values into the T matrix which will be used to relate the 3 term
% connection coefficients.
for l = -nm2:nm2
    for m = -nm2:nm2
        if(abs(l-m) <= nm2)
            row = 1 + row;
            col = 0;
            for n = -nm2:nm2
                for s = -nm2:nm2
                    if(abs(n-s) <= nm2)
                        col = 1 + col;
                        q = 2*l - n;
                        r = 2*m - s;
                        if((abs(q) <= nm1)&&(abs(r)<=nm1))
                            T(row,col) = A(q+N,r+N);
                        end
                    end
                end
            end
        end
    end
end


% Subtract the diagonal part from T
T = T-((1/(2.^(d-1))).*eye(size(T)));

% 
% Compute the moments
%

% We are going to have three sets of moments that we are going to need to
% index into our linear system.  But first we must compute them.  To do
% this we must set the first scaling function to have a zero derivative
% term like: \phi^0 \phi^{d_2} \phi^{d_3}.  In this form we need to compute
% the moments:

% Everything from here down is borrowed from Jordi Besora.  As is, it is 
% not very efficient and should be improved.  I will continue to work with
% it.

% Calculate the moments
dmax = max(d2,d3);
Mom = moments(N,level,dmax,h);

% These are some values that we will use to put the moments into our T
% matrix
sizes = 2*(nT+d);
offset = sizes/2;
ini = offset + 1;
Lm = 2*N-3; % Lm = 2*N-3 = 9;
Lm1 = Lm + 1; % Lm1 = 10;
Q = Lm1-N+1; % Q = 5;
Lm100 = Lm + offset; %
N100 = N + offset; %

L = zeros(1,sizes);
Li = L;
L1=2-N:N-2; % L = [-4 ... 0 ... 4];
L(ini:Lm100) = L1;
Li(ini:Lm100) = L(ini:Lm100) + N-1; % Li = [1 ... 9];

a=zeros(1,sizes); % The Daubechies coefficients.
% The DN wavelet coefficients according to "Numerical Recipes"
% a/h = sqrt(2) => a = sqrt(2) * h
a(ini:N100) = daub_coefficients(N).*sqrt(2);


nm = zeros(1,sizes);
n_aux =zeros(1,Lm);
nt = zeros(1,sizes);
for m = 1:nm2 % 1...4
    n_aux(m) = 1 + 2*(N-2) + L1(m);
end
for m = nm1:Lm
    n_aux(m) = 1 + 2*(N-2) - L1(m);
end
nm(ini:Lm100) = n_aux;
clear n_aux;
for m = ini:Lm100
    nt(m) = nt(m-1) + nm(m);
end


if (mod(d2,2)==0)
    m = 0;
else
    m = 1;
end


for b = 1:d2
    i = nT + b;
    for l = ini+m:Lm100 % 1+m:9 (-4+m:4)
        j = 1 + L(l) - m + (N-2) + nt(ini+N-2+m-1);
        T(i,j) = Mom(b,l);
        % mssg = ['A(' num2str(i) ',' num2str(j) ') = ' num2str(A(i,j),18)];
        % disp(mssg);
    end
end

if (mod(d3,2)==0)
    l = 0;
else
    l = 1;
end

for c = 1:d3
    i = nT + d2 + c;
    for m = ini+l:ini+Q-2 % 1+l:4 (-4+l:-1)
        j = 1 + l + (N-2) + nt(m-1);
        T(i,j) = Mom(c,m);
        % mssg = ['A(' num2str(i) ',' num2str(j) ') = ' num2str(A(i,j),18)];
        % disp(mssg);
    end
    for m = ini+Q-1:Lm100 % 5:9 (0:4)
        j = 1 + l - L(m) + (N-2) + nt(m-1);
        T(i,j) = Mom(c,m);
        % mssg = ['A(' num2str(i) ',' num2str(j) ') = ' num2str(A(i,j),18)];
        % disp(mssg);
    end
end

i = nT + d +1;
for m = ini:ini+Q-2 % 1:4 (-4:-1)
    for l = ini:Lm100+L(m) % 1:9-m (-4:4-m)
        j = 1 + L(l) + (N-2) + nt(m-1);
        T(i,j) = Mom(d2+1,l) * Mom(d3+1,m);
        % mssg = ['A(' num2str(i) ',' num2str(j) ') = ' num2str(A(i,j),18)];
        % disp(mssg);
    end
end
for m = ini+Q-1:Lm100 % 5:9 (0:4)
    for l = ini+L(m):Lm100 % 1+m:9 (-4+m:4)
        j = 1 + L(l) - L(m) + (N-2) + nt(m-1);
        T(i,j) = Mom(d2+1,l) * Mom(d3+1,m);
        % mssg = ['A(' num2str(i) ',' num2str(j) ') = ' num2str(A(i,j),18)];
        % disp(mssg);
    end
end


% Form the right hand side of the matrix
rhs = zeros(nT+d+1,1);
fac1 = 1;
fac2 = 1;
if(d2>0)
  k = (2.*d2) - 1;
  fac1 = 2.^k;
end
if(d3>0)
  k = (2.*d3) - 1;
  fac2 = 2.^k;
end
rhs(nT+d+1) = fac1.*fac2.*(factorial(d2).*factorial(d3))./(sqrt(2.^level));

% Solve the system
cc_s = T\rhs;

% Determine if the system is rank deficient
if(rank(T)<min(size(T)))
    sucess = 0;
else
    sucess = 1;
end

cc = zeros(Lm,Lm); % 9x9

i = 0; % max col index for lmbd
j = 1; % index for lmbd_s
for l = 1:Q % The first half
    for m = 1:Q+i
        cc(l,m) = cc_s(j);
        j = j+1;
    end
    i = i+1;
end
i = Q-2;
for l = Q+1:Lm % The second half
    for m = Q-i:Lm
        cc(l,m) = cc_s(j);
        j = j+1;
    end
    i = i-1;
end

% This is the wrong way to do it but ohh well.  Take the matrix we have
% just formed and vectorize it.
cc = cc(:);



             