% This script is intended to write a file to be used by a fortran program
% in order to determine the connection coefficients for a given set of
% inputs

% Write CC2 or CC3 or both
w_write = 'CC2';


if(strcmp(w_write,'CC2') || strcmp(w_write,'both'))
    % First open the file for writing
    FID = fopen('CC_2.f90','w');

    % Now put some of theimportant stuff at the top
    fprintf(FID,'subroutine CC_2(N,level,d,nO,Omega)\n');
    fprintf(FID,'! This subroutine stores the two term connection coefficient data\n');
    fprintf(FID,'! dependant on the scaling function genus, the resolution and the\n');
    fprintf(FID,'! value of the sum of the two derivatives.\n');
    fprintf(FID,'implicit none\n');

    % Now do some initialization stuff to make sure our output vector is good
    % to go
    fprintf(FID,'integer (kind=4) N\n');
    fprintf(FID,'integer (kind=4) level\n');
    fprintf(FID,'integer (kind=4) d\n');
    fprintf(FID,'integer (kind=4) nO\n');
    fprintf(FID,'real (kind=8) Omega(1:nO)\n\n\n');
    
    fprintf(FID,'write(*,*) ''Now extracting the connection Coefficients''\n\n');

    % Now we have all we need to go ahead and start writing out the connection
    % coefficients
    count = 1;
    for N=4:2:20
        for level=1:10
            for d=1:3
                fprintf('Writing CCs for: N=%i, level=%i and d=%i\n\n',N,level,d);
                % Calculate the connection coefficients
                [~,~,O,~] = connection_coeffs(N,d,0,level);

                nO = length(O);

                % Check to determine the if statement we need to write
                if(count == 1)
                    fprintf(FID,'\tif((N==%i) .and. (level==%i) .and. (d==%i)) then\n',N,level,d);
                else
                    fprintf(FID,'\telse if((N==%i) .and. (level==%i) .and. (d==%i)) then\n',N,level,d);
                end

                % Now go through and print the data
                fprintf(FID,'\t\tOmega = (/ %1.10e, %1.10e, &\n',O(1),O(2));
                for i=3:2:(nO-2)
                    fprintf(FID,'\t\t\t%1.10e, %1.10e, &\n',O(i),O(i+1));
                end
                fprintf(FID,'\t\t\t%1.10e /)\n',O(nO));
                count = count +1;
            end
        end
    end


    % Then print the concluding stuff at the end of the file
    fprintf(FID,'\tendif\n');
    fprintf(FID,'\treturn\n');
    fprintf(FID,'end\n');

    % Close the cc2 file
    fclose(FID);
end



if(strcmp(w_write,'CC3') || strcmp(w_write,'both'))
    % First open the file for writing
    FID = fopen('CC_3.f90','w');

    % Now put some of theimportant stuff at the top
    fprintf(FID,'subroutine CC_3(N,level,d2,d3,nO,Omega,sucess)\n');
    fprintf(FID,'! This subroutine stores the three term connection coefficient data\n');
    fprintf(FID,'! dependant on the scaling function genus, the resolution and the\n');
    fprintf(FID,'! value of the two derivatives.\n');

    % Now do some initialization stuff to make sure our output vector is good
    % to go
    fprintf(FID,'integer (kind=4) N\n');
    fprintf(FID,'integer (kind=4) level\n');
    fprintf(FID,'integer (kind=4) d2\n');
    fprintf(FID,'integer (kind=4) d3\n');
    fprintf(FID,'integer (kind=4) nO\n');
    fprintf(FID,'real (kind=8) Omega(1:nO\n\n\n');
    
    fprintf(FID,'! This flag lets us know whether the connection coefficients\n');
    fprintf(FID,'! can be trusted. If the matrix we solve for is rank defficient\n');
    fprintf(FID,'! a zero will be returned. Otherwise a sucessful computation\n');
    fprintf(FID,'! is noted as a one');
    fprintf(FID,'integer (kind=4) sucess\n');

    % Now we have all we need to go ahead and start writing out the connection
    % coefficients
    count = 1;
    for N=4:2:20
        for level=1:10
            for d2=1:2
                for d3=1:2
                    fprintf('Writing CCs for: N=%i, level=%i, d2=%i and d3=%i\n\n',N,level,d2,d3);
                    % Calculate the connection coefficients
                    [O,sucess] = connection_coeffs3(N,0,d2,d3,level);

                    nO = length(O);

                    % Check to determine the if statement we need to write
                    if(count == 1)
                        fprintf(FID,'\tif((N==%i) .and. (level==%i) .and. (d==%i)) then\n',N,level,d);
                    else
                        fprintf(FID,'\telseif((N==%i) .and. (level==%i) .and. (d==%i)) then\n',N,level,d);
                    end
                    
                    % Now go through and print the data
                    fprintf(FID,'\t\tOmega = (/ %1.10e, %1.10e, &\n',O(1),O(2));
                    for i=3:2:(nO-2)
                        fprintf(FID,'\t\t\t%1.10e, %1.10e, &\n',O(i),O(i+1));
                    end
                    fprintf(FID,'\t\t\t%1.10e /)\n',O(nO));
                    
                    % If the matrix is rank deficient then note that the
                    % answer is not to be trusted
                    fprintf(FID,'sucess = %i\n\n',sucess);
                    
                    count = count +1;
                end
            end
        end
    end


    % Then print the concluding stuff at the end of the file
    fprintf(FID,'\tendif\n');  
    fprintf(FID,'\treturn\n');
    fprintf(FID,'end\n');

    % Close the cc2 file
    fclose(FID);
end
            
            