% This program will attempt to find the connection coefficients of our wavelet
% basis functions according to the paper by Chen and then corrected by
% Zhang


function [A,b,Omega] = Zhang_Connection_coefficients(L,d1,d2,level)


% The derivative order
n = d1 + d2;

% Pull in the daubechies coefficients and store them as a values which
% should sum to 2
a = sqrt(2).*daub_coefficients(L);


% Define the Matrix D which we wish to solve for
D =  zeros(L,L-1);

% Loop over all the elements in l and m
for l=1:L-1
    for m=1:L-1
        
        % Zero the temporary variables
        temp1 = 0;
        temp2 = 0;
        
        % Now loop over the i's and j's to sum them to the l,m element
        for i=0:L-1
            for j=0:L-1
                
                % Check if the values fit our equation
                if(((2*(l-1))+i-j) == (((-1)^2)*(m-1)))
                    temp1 = temp1 + a(i+1).*a(j+1);
                end
                
                if(((2*(l-1))+i-j) == (((-1)^3)*(m-1)))
                    temp2 = temp2 + a(i+1).*a(j+1);
                end
            end
        end
        
        D(l,m) = (2^(n-1)).*(temp1 + ((-1)^n).*temp2);
        
        % Subtract the rhs of the equation from the D matrix
        if(l == m)
            D(l,m) = D(l,m) - 1;
        end
        
    end
end

% Add in the final row of the D matrix
for k=0:L-2
    D(L,k+1) = (k.^n)./(2^(-level./2));
end

% % Once D is fomulated we need to solve the eigenvalue problem
% [X,v] = eig(D);

% % Convert our matrix of eigenvalues into a vector
% v = spdiags(v);

% % Extract the column where the eigenvalue is at unity
% temp = X(:,abs(1-v)==min(abs(1-v)));

% Define our right hand side
b = zeros(L,1);
b(L) = factorial(n)./(2^(level./2));

% solve the linear system
temp = D\b;
%Omega = temp;
A = D;

% Now we need to formulate the connection coefficients using
% assumption (9) in Zhang et al
nO = (2.*(L-1))-1;
midO = ceil(nO./2);
Omega = zeros(nO,1);

% Set the middle value
Omega(midO) = temp(1);

for k = 1:midO-1
    Omega(midO+k) = temp(k+1);
    Omega(midO-k) = ((-1).^n).*temp(k+1);
    %Omega(midO-k) = temp(k+1);
end

% Loop over all the values to



% Check to make sure the normalization worked
% normal = 0;
% for k=0:L-2
    % normal = normal + (k.^n).*(temp(k+1));
% end





