% This script forms the matrix that will be used to display the connection
% coefficients in mi thesis

clear
clc
close all


% For the daubechies wavelets we want to look at
for D=6:2:10
    
    % Setup the Omega matrix that will contain our connection coeffs
    nO = (2*D) - 3;
    O = zeros(nO,6);
    i=1;
    
    % For all the levels we want to generate
    for level = 1:2:5
    
        % Find the connection coeffs for 1 deriv
        [~,~,o,~] = connection_coeffs(D,1,0,level);
        O(:,i) = o';
                
        % And then for 2 derivs
        [~,~,o,~] = connection_coeffs(D,2,0,level);
        O(:,i+3) = o';
        i = i + 1;
        
    end
    
    
    % Now write the info for the table
    fprintf('\\begin{table}[H]\n');
    fprintf('\\begin{center}\n');
    fprintf('\\begin{tabular}{| c | c | c | c | c | c |}\n');
    fprintf('\\hline\n');
    fprintf('\\cline{2-6}\n');
    fprintf('& \\multicolumn{2}{ c | }{$d = 1$} & \\multicolumn{3}{  c | }{$d = 2$} \\\\\n');
    fprintf('\\hline\n');
    fprintf('$\\Gamma$ & $m = 1$ & $m = 3$ & $m = 1$ & $m = 2$ & $m = 3$ \\\\\n');
    fprintf('\\hline\n');
    
    % Determine the shift
    k = -floor(nO./2);
    
    % Then loop over all the coefficients we generated
    for i=1:nO
        
        % Print the shift we are on
        fprintf('$k = %i$',k);
        
        % THen print all the values in the row
        for j=1:6
            
            % Skip 1deriv m=2
            if(j == 2)
                continue;
            end
            fprintf(' & %e',O(i,j));
        end
        
        fprintf('\\\\\n');
        fprintf('\\hline\n');
        
        k = k+1;
    end
    
    % Conclude the table
    fprintf('\\end{tabular}\n');
    fprintf('\\caption{Connection Coefficients for $D%i$}\n',D);
    fprintf('\\label{connection_coefficients%i}\n',D);
    fprintf('\\end{center}\n');
    fprintf('\\end{table}\n');
end
    
    



