% This script will formulate the T mapping matrix defined in the Nielson
% paper so that we can map our rhs function to the appropriate scaling
% function space

function [cij] = map_to_scaling_fn_space(fx,N,level,Nx)

% Use the geometry routine to get the upper and lower bounds of the system
[xl,xr,~,~] = geometry();

% Nx is the number of functions we have to define the function on our
% domain
Nx = ((xr-xl)/(1/(2^level))) + 1;

%
% First find the values of the scaling function at the first dyadic points
%

% Get the scaling function coefficients
h = daub_coefficients(N);

% Put the A matrix together so that we can determine the values of phi at
% the integers using the eigenvector method.
A = zeros(N,N);

% i is for each row in the A matrix
for i=1:N
    
    % k is for each of the values within the wavelet calculation
    for k=1:N
        
        % This is the value that is in the phi function on the rhs
        x_rhs = (2).*(i-1) - (k-1);
        
        % If the x value is within our range then we can put it in A
        if(x_rhs >= 0 && x_rhs <= N-1)
            A(i,x_rhs+1) = h(k).*(sqrt(2)   );
        end
    end
end

% Now that we have A we can calculate the eigenvector and find the values
% of our scaling function at the integer values
[v,d] = eig(A);

% Multiply the eigenvalues/eigenvectors by sqrt(2)
d = spdiags(d);
v = sqrt(2).*v;

% Extract the eigenvector where the eigenvalue is equal to 1
phi = -v(:,abs(1-d)==min(abs(1-d)))';

% Flip the vector sothat we can directly index it into the T matrix
phi = fliplr(phi);

% Now that we have the values of the scaling function at the first dyadic
% points we can set up the T or mapping matrix
T = zeros(Nx,Nx);
 
% Loop over all the equations that we are using so that we can formulate
% each row of the T matrix
for i=1:Nx
    
    % The number of values that we have to worry about is N-1 or all the
    % values of the scaling function at the integers in the domain
    
    % If we are less than N-1 then we have to do something tricky on how
    % to arrange the T matrix
    if(i<=N)
        T(i,1:i) = phi(N-i+1:N);
        T(i,Nx-(N-i)+1:Nx) = phi(1:N-i);
    else
        T(i,i-N+1:i) = phi;
    end
end


% Now that we have T we can solve for c
cij = T\fx;








